using NewUpdateDialogSpace;

using System.Collections;

using UnityEngine;

namespace MinecraftSwords
{
    internal class SwordsExtetions
    {
        public static void RegisterSword(string order, string name, string spritename, string desc, float sharpness = 4f, string properties = null)
        {
            ModAPI.Register(
            new Modification()
            {
                OriginalItem = ModAPI.FindSpawnable("Sword"),
                NameOverride = name + Mod.ModTag,
                DescriptionOverride = desc,
                NameToOrderByOverride = $"WeaponsSwordsMaterial{order}",
                CategoryOverride = ModAPI.FindCategory("Int`s Minecraft Mod"),
                ThumbnailOverride = ModAPI.LoadSprite($"Thumbnails/{spritename}.png"),
                AfterSpawn = (Instance) =>
                {
                    PhysicalBehaviour phys = Instance.GetComponent<PhysicalBehaviour>();
                    if (properties != null)
                    {
                        phys.Properties = ModAPI.FindPhysicalProperties($"{properties}");
                    }

                    phys.spriteRenderer.sprite = ModAPI.LoadSprite($"Textures/{spritename}.png");

                    Collider2D.Destroy(phys.colliders[1]);

                    BoxCollider2D Garda = Instance.AddComponent<BoxCollider2D>();
                    Garda.size = new Vector2(0.36f, 0.08f);
                    Garda.offset = new Vector2(0f, -0.255f);

                    BoxCollider2D Handle = Instance.AddComponent<BoxCollider2D>();
                    Handle.size = new Vector2(0.1294f, 0.35f);
                    Handle.offset = new Vector2(0f, -0.41f);

                    BoxCollider2D Blade = Instance.GetComponent<BoxCollider2D>();
                    Blade.size = new Vector2(0.08f, 0.85f);
                    Blade.offset = new Vector2(0f, 0.19f);

                    SwordBehaviour swordBehaviour = Instance.GetOrAddComponent<SwordBehaviour>();
                    swordBehaviour.PhysicalBehaviour = phys;
                    swordBehaviour.Collider = Blade;
                    swordBehaviour.Sharpness = sharpness;

                    NewUpdateDialog.TryCreateNewUpdateDialog();
                }
            });
        }
    }
    public class Mod
    {
        public const string ModTag = " [IMM]"; //* Whilst not required, modded items should contain 'tags' at the end of their names to prevent errors in which two mods have an item of the same name.

        public static void Main()
        {
            ModAPI.RegisterCategory("Int`s Minecraft Mod", "", ModAPI.LoadSprite("Thumbnails/Category.png"));

            SwordsExtetions.RegisterSword("A", "Minecraft Wooden Sword", "Wooden Sword", "Sword made of wood. This sword cannot stab entities. When hit, it remove a certain amount of health to all limbs. Damage depends on sharpness. Sharpness: 1", 1f, "Wood");
            SwordsExtetions.RegisterSword("B", "Minecraft Stone Sword", "Stone Sword", "Sword made of stone. When hit, it remove a certain amount of health to all limbs. Damage depends on sharpness. Sharpness: 2", 2f);
            SwordsExtetions.RegisterSword("C", "Minecraft Golden Sword", "Gold Sword", "Sword made of gold. When hit, it remove a certain amount of health to all limbs. Damage depends on sharpness. Sharpness: 3.5", 3.5f);
            SwordsExtetions.RegisterSword("D", "Minecraft Iron Sword", "Iron Sword", "Sword made of iron. When hit, it remove a certain amount of health to all limbs. Damage depends on sharpness. Sharpness: 3", 3f);
            SwordsExtetions.RegisterSword("E", "Minecraft Diamond Sword", "Diamond Sword", "Sword made of diamond. When hit, it remove a certain amount of health to all limbs. Damage depends on sharpness. Sharpness: 4.5", 4.5f);
            SwordsExtetions.RegisterSword("F", "Minecraft Netherite Sword", "Netherite Sword", "Sword made of netherite. When hit, it remove a certain amount of health to all limbs. Damage depends on sharpness. Sharpness: 7", 7f);
        }
    }
    public class SwordBehaviour : MonoBehaviour
    {
        [SkipSerialisation]
        public float Sharpness;
        [SkipSerialisation]
        public float Knockback = 0.4f;
        [SkipSerialisation]
        public float KnockbackGravityMult = 0.2f;

        [SkipSerialisation]
        public PhysicalBehaviour PhysicalBehaviour;
        [SkipSerialisation]
        public Collider2D Collider;
        [SkipSerialisation]
        private static readonly Color HitColor = new Color(1f, 0.30f, 0.30f);

        [SkipSerialisation]
        private PersonBehaviour LastHitPerson;

        protected void OnCollisionEnter2D(Collision2D collision)
        {
            if (collision.otherCollider != Collider || !collision.collider.TryGetComponent<LimbBehaviour>(out LimbBehaviour limb))
            {
                return;
            }

            float power = collision.relativeVelocity.magnitude;
            if (!(5f < power) || !limb.Person.IsAlive())
            {
                return;
            }

            float damage = power * Sharpness * 0.2f;
            Vector2 knockback = (Vector2.ClampMagnitude(PhysicalBehaviour.GetPreviousVel(), 1f) - (Physics2D.gravity * KnockbackGravityMult)) * Knockback;
            Debug.Log("knockback: " + knockback);

            bool setColor = LastHitPerson != limb.Person;
            for (int i = 0; i < limb.Person.Limbs.Length; i++)
            {
                LimbBehaviour limb2 = limb.Person.Limbs[i];
                limb2.Health -= damage;
                limb2.PhysicalBehaviour.rigidbody.AddForce(knockback * limb2.PhysicalBehaviour.rigidbody.mass, ForceMode2D.Impulse);

                if (setColor)
                {
                    limb2.PhysicalBehaviour.spriteRenderer.color = HitColor;
                }
            }
            if (setColor)
            {
                StartCoroutine(ResetLimbsColor(limb.Person));
            }
        }
        private IEnumerator ResetLimbsColor(PersonBehaviour person)
        {
            LastHitPerson = person;
            yield return new WaitForSeconds(0.2f);

            for (int i = 0; i < person.Limbs.Length; i++)
            {
                person.Limbs[i].PhysicalBehaviour.spriteRenderer.color = Color.white;
            }
            LastHitPerson = null;
        }
    }
}
// Originally uploaded by 'Int Team'. Do not reupload without their explicit permission.
